/**
 * Unified Page Component
 * 
 * A unified component for rendering all types of pages (main, clients, cases)
 * using a consistent block-based content structure.
 * 
 * Features:
 * - Dynamic content loading from API
 * - Fallback content support
 * - Authentication-aware content fetching
 * - Smooth loading transitions
 * - Error handling with user-friendly messages
 * - SEO-friendly structure
 * - Hierarchical page support (parent/child URLs)
 * - Real-time preview updates via PostMessage API
 */
import React, { useEffect, useState, useMemo } from 'react';
import { useParams, useLocation, useNavigate } from 'react-router-dom';
import { ContentBlock, renderBlock } from '../components/blocks/BlockRenderer';
import { PerformanceOptimizedRenderer } from '../components/blocks/PerformanceOptimizedRenderer';
import { useHeroImagePreload } from '../hooks/useHeroImagePreload';
import { useEarlyHeroPreload, useAggressivePreload } from '../hooks/useEarlyHeroPreload';

import { apiClient } from '../api/client';
import { useAuth } from '../contexts/AuthContext';
import { useLanguage, getBasePath } from '../components/LanguageRouter';
import { getLocalizedContent } from '../utils/languageUtils';
import { usePageLayoutSettings } from '../hooks/usePageLayoutSettings';
import { previewListener } from '../utils/previewListener';
import NotFoundPage from './NotFoundPage';
import LayoutDebugger from '../components/LayoutDebugger';
import './Page.css';

// Define interfaces for different page types
interface BasePageData {
  _id?: string;
  identifier: string;
  title: string;
  overview: ContentBlock[] | null;
  published?: boolean;
  updatedAt?: string;
}

interface CasePageData extends BasePageData {
  slug: string;
}

interface MainPageData extends BasePageData {
  identifier: 'main';
}

interface ClientsPageData extends BasePageData {
  identifier: 'clients';
}

interface CasesPageData extends BasePageData {
  identifier: 'cases';
}

interface CustomPageData extends BasePageData {
  // Custom pages can have any identifier
}

type PageData = CasePageData | MainPageData | ClientsPageData | CasesPageData | CustomPageData;

// Helper function to determine if a path segment is likely a page identifier
const isSystemRoute = (segment: string): boolean => {
  return ['clients', 'cases', 'admin', 'en'].includes(segment);
};

// Helper function to determine page type and identifier from URL
const analyzePageFromUrl = (pathname: string) => {
  const basePath = getBasePath(pathname);
  const segments = basePath.split('/').filter(Boolean); // Remove empty segments
  
  // Root page
  if (segments.length === 0) {
    return { pageType: 'main', identifier: 'main', isHierarchical: false };
  }
  
  // Single segment
  if (segments.length === 1) {
    const segment = segments[0];
    
    // System pages
    if (segment === 'clients') return { pageType: 'clients', identifier: 'clients', isHierarchical: false };
    if (segment === 'cases') return { pageType: 'cases', identifier: 'cases', isHierarchical: false };
    
    // Custom single page
    return { pageType: 'customPage', identifier: segment, isHierarchical: false };
  }
  
  // Two segments - these are now hierarchical pages in the unified system
  if (segments.length === 2) {
    const [first, second] = segments;
    
    // All two-segment URLs are now hierarchical pages (parent/child)
    // Legacy case routes will be handled by redirects at the API level
    return { pageType: 'hierarchicalPage', identifier: second, parentIdentifier: first, isHierarchical: true };
  }
  
  // More than 2 segments - not supported yet
  return { pageType: 'unknown', identifier: null, isHierarchical: false };
};

const Page: React.FC = () => {
  // Get all possible URL parameters
  const { 
    caseId, 
    pageId, 
    parentId, 
    childId 
  } = useParams<{ 
    caseId: string; 
    pageId: string; 
    parentId: string; 
    childId: string; 
  }>();
  
  const location = useLocation();
  const navigate = useNavigate();
  const { user } = useAuth();
  const { currentLanguage } = useLanguage();
  
  // Extract version and preview parameters from URL
  const urlParams = new URLSearchParams(location.search);
  const versionId = urlParams.get('version');
  const previewKey = urlParams.get('preview');
  const isPreviewMode = !!previewKey;
  
  // State for the fetched page data, loading status, and errors
  const [pageData, setPageData] = useState<PageData | null>(null);
  const [isLoading, setIsLoading] = useState<boolean>(true);
  const [error, setError] = useState<string | null>(null);
  const [usingFallback, setUsingFallback] = useState<boolean>(false);

  // Analyze the current URL to determine page type and identifier
  const urlAnalysis = useMemo(() => {
    return analyzePageFromUrl(location.pathname);
  }, [location.pathname]);

  // Determine final page type and identifier based on URL analysis and route parameters
  const { pageType, pageIdentifier } = useMemo(() => {
    const { pageType: analyzedType, identifier, parentIdentifier, isHierarchical } = urlAnalysis;
    
    // Handle different route parameter combinations
    if (caseId) {
      // Legacy case route (/cases/:caseId or /clients/:caseId)
      return { pageType: 'case', pageIdentifier: caseId };
    } else if (parentId && childId) {
      // Hierarchical route (/:parentId/:childId)
      return { pageType: 'hierarchicalPage', pageIdentifier: childId };
    } else if (pageId) {
      // Single page route (/:pageId)
      return { pageType: 'customPage', pageIdentifier: pageId };
    } else {
      // Use URL analysis
      return { pageType: analyzedType, pageIdentifier: identifier || 'main' };
    }
  }, [urlAnalysis, caseId, pageId, parentId, childId]);

  // Log page analysis only in development
  if (import.meta.env.DEV) {
    console.log('Page URL Analysis:', {
      pathname: location.pathname,
      basePath: getBasePath(location.pathname),
      urlAnalysis,
      params: { caseId, pageId, parentId, childId },
      finalResult: { pageType, pageIdentifier }
    });
  }

  // Load page-specific layout settings first
  const { loading: layoutLoading } = usePageLayoutSettings(pageIdentifier);

  // Critical performance optimization: start preloading hero images immediately
  // This happens before page data is loaded, providing maximum LCP benefit
  useEarlyHeroPreload();
  useAggressivePreload();

  // Initialize preview listener for real-time updates when in preview mode
  useEffect(() => {
    if (isPreviewMode) {
      console.log('🎯 Preview mode detected - setting up ENHANCED live preview listener');
      
      // Signal to parent that preview is ready after a short delay to ensure DOM is loaded
      const signalPreviewReady = () => {
        console.log('📡 Sending PREVIEW_READY signal to parent window');
        if (window.parent !== window) {
          window.parent.postMessage({
            type: 'PREVIEW_READY',
            payload: { 
              timestamp: Date.now(),
              url: window.location.href 
            },
            timestamp: Date.now()
          }, window.location.origin);
        }
      };
      
      // Wait for DOM to be fully ready before signaling
      if (document.readyState === 'complete') {
        // If already loaded, signal immediately
        setTimeout(signalPreviewReady, 100);
      } else {
        // Otherwise wait for load event
        window.addEventListener('load', () => {
          setTimeout(signalPreviewReady, 100);
        });
      }
      
      // Cache for accumulating block updates
      const blockUpdateCache = new Map<string, any>();
      
      // Block highlighting functions for live hover preview
      const highlightBlock = (blockId: string) => {
        // Remove any existing highlights first
        document.querySelectorAll('.live-highlight').forEach(el => {
          el.classList.remove('live-highlight');
        });
        
        // Find and highlight the target block
        const targetElement = document.querySelector(`[data-block-id="${blockId}"]`);
        if (targetElement) {
          targetElement.classList.add('live-highlight');
          console.log('✅ Block highlighted successfully:', blockId);
        } else {
          console.log('❌ Block element not found for highlighting:', blockId);
        }
      };
      
      const unhighlightBlock = (blockId: string) => {
        const targetElement = document.querySelector(`[data-block-id="${blockId}"]`);
        if (targetElement) {
          targetElement.classList.remove('live-highlight');
          console.log('✅ Block highlight removed successfully:', blockId);
        }
        
        // Also remove any other highlights as cleanup
        document.querySelectorAll('.live-highlight').forEach(el => {
          el.classList.remove('live-highlight');
        });
      };
      
      // Highlight insert position functions for add block zones - IMPROVED VERSION
      const highlightInsertPosition = (insertIndex: number, isGridPosition = false) => {
        console.log('🎯 Insert position highlight requested for index:', insertIndex, isGridPosition ? '(grid)' : '(root)');
        
        // Clean up first
        unhighlightInsertPosition();
        
        if (isGridPosition) {
          // Skip grid highlighting to avoid global grid highlights
          console.log('🎯 Skipping grid position highlight to avoid global highlighting');
          return;
        }
        
        // Find the content blocks container
        const container = document.querySelector('.content-blocks-container');
        if (!container) {
          console.log('❌ Content blocks container not found');
          return;
        }
        
        // Find all block wrapper elements (the actual layout elements)
        const blockWrappers = container.querySelectorAll('.block-wrapper');
        console.log('🔍 Found block wrappers:', blockWrappers.length, 'insertIndex:', insertIndex);
        
        // Create highlight line element
        const highlightLine = document.createElement('div');
        highlightLine.className = 'insert-position-highlight-line';
        highlightLine.style.cssText = `
          position: absolute;
          left: 0;
          right: 0;
          height: 3px;
          background: linear-gradient(90deg, #3b82f6, #8b5cf6);
          border-radius: 2px;
          z-index: 1000;
          opacity: 0.8;
          animation: insertHighlightPulse 1.5s ease-in-out infinite;
          box-shadow: 0 0 8px rgba(59, 130, 246, 0.4);
          pointer-events: none;
        `;
        
        // Determine where to place the highlight line
        let targetPosition = { top: 0, containerRelative: true };
        
        if (insertIndex === 0 && blockWrappers.length > 0) {
          // Insert at the beginning - highlight above first block
          const firstBlock = blockWrappers[0] as HTMLElement;
          const containerRect = container.getBoundingClientRect();
          const blockRect = firstBlock.getBoundingClientRect();
          targetPosition.top = blockRect.top - containerRect.top - 8;
        } else if (insertIndex > 0 && insertIndex < blockWrappers.length) {
          // Insert between blocks - highlight between blocks
          const blockBefore = blockWrappers[insertIndex - 1] as HTMLElement;
          const blockAfter = blockWrappers[insertIndex] as HTMLElement;
          const containerRect = container.getBoundingClientRect();
          const beforeRect = blockBefore.getBoundingClientRect();
          const afterRect = blockAfter.getBoundingClientRect();
          
          // Place highlight in the middle of the gap
          const gapMiddle = (beforeRect.bottom + afterRect.top) / 2;
          targetPosition.top = gapMiddle - containerRect.top - 1.5; // Center the 3px line
        } else if (blockWrappers.length > 0) {
          // Insert at the end - highlight below last block
          const lastBlock = blockWrappers[blockWrappers.length - 1] as HTMLElement;
          const containerRect = container.getBoundingClientRect();
          const blockRect = lastBlock.getBoundingClientRect();
          targetPosition.top = blockRect.bottom - containerRect.top + 8;
        } else {
          // No blocks yet - highlight at the top of container
          targetPosition.top = 20;
        }
        
        // Position the highlight line relative to container
        highlightLine.style.top = `${targetPosition.top}px`;
        
        // Add to container with relative positioning
        (container as HTMLElement).style.position = 'relative';
        container.appendChild(highlightLine);
        
        console.log('✅ Insert position highlight added at position:', targetPosition.top, 'for index:', insertIndex);
      };
      
      const unhighlightInsertPosition = () => {
        console.log('🧹 Cleaning up insert position highlights');
        
        // Remove highlight line elements
        document.querySelectorAll('.insert-position-highlight-line').forEach(el => {
          if (el.parentNode) {
            el.parentNode.removeChild(el);
          }
        });
        
        console.log('✅ Insert position cleanup completed');
      };
      
      // Helper function for complete layout rebuild - REACT-FRIENDLY VERSION
      const rebuildCompleteLayout = (newContentBlocks: any[]) => {
        console.log('🔄 Triggering React layout rebuild instead of direct DOM manipulation');
        
        // Instead of direct DOM manipulation, trigger a React re-render
        // by dispatching a custom event that the parent component can listen to
        const rebuildEvent = new CustomEvent('reactLayoutRebuild', {
          detail: { contentBlocks: newContentBlocks }
        });
        window.dispatchEvent(rebuildEvent);
        
        console.log('✅ React layout rebuild event dispatched');
      };

      // Layout reordering handler functions
      const handleBlockReordering = (layoutData: any) => {
        const { contentBlocks, movedBlockId, fromIndex, toIndex } = layoutData;
        console.log('🔄 Reordering blocks - rebuilding complete layout for sync');
        console.log('🔍 New layout order:', contentBlocks?.map((b: any) => `${b.id?.substring(0, 8)}...`));
        
        // Instead of moving individual blocks, rebuild the entire layout to ensure perfect sync
        const container = document.querySelector('.content-blocks-container');
        if (!container) {
          console.log('❌ No content-blocks-container found');
          return;
        }
        
        // If no contentBlocks provided, fall back to old behavior
        if (!contentBlocks || !Array.isArray(contentBlocks)) {
          console.log('⚠️ No contentBlocks array provided, falling back to individual move');
          // Fallback to the old single-block moving logic would go here
          return;
        }
        
        // Get all current block wrappers and create a map
        const currentBlockElements = new Map();
        Array.from(container.children).forEach(child => {
          const blockId = child.querySelector('[data-block-id]')?.getAttribute('data-block-id');
          if (blockId) {
            currentBlockElements.set(blockId, child);
          }
        });
        
        console.log('🔍 Current blocks in DOM:', Array.from(currentBlockElements.keys()).map(id => `${id.substring(0, 8)}...`));
        
        // Remove all blocks from container (but keep them in memory)
        currentBlockElements.forEach(element => {
          element.remove();
        });
        
        // Add blocks back in the correct order according to contentBlocks
        let successCount = 0;
        contentBlocks.forEach((block: any, index: number) => {
          const element = currentBlockElements.get(block.id);
          if (element) {
            container.appendChild(element);
            successCount++;
          } else {
            console.log('⚠️ Block element not found in DOM:', `${block.id?.substring(0, 8)}...`);
          }
        });
        
        if (successCount === contentBlocks.length) {
          console.log('✅ Complete layout rebuild successful! All blocks reordered.');
        } else {
          console.log(`⚠️ Partial layout rebuild: ${successCount}/${contentBlocks.length} blocks reordered.`);
        }
      };
      
      const handleGridColumnReordering = (layoutData: any) => {
        console.log('🔄 Grid column reordering:', layoutData);
        
        const { gridBlockId, rowIndex, fromColumnIndex, toColumnIndex, contentBlocks } = layoutData;
        
        // Find the grid block in the DOM
        const gridElement = document.querySelector(`[data-block-id="${gridBlockId}"]`);
        if (!gridElement) {
          console.log('❌ Grid block element not found:', gridBlockId);
          return;
        }
        
        console.log('🔍 Found grid element, rebuilding with new column order');
        
        // For grid column reordering, we need to rebuild the entire layout
        // since column structure changes are complex
        rebuildCompleteLayout(contentBlocks);
      };
      
      const handleGridItemMove = (layoutData: any) => {
        console.log('🔄 Grid item move:', layoutData);
        
        const { 
          movedItemId, 
          sourceGridId, 
          sourceColumnIndex, 
          targetGridId, 
          targetColumnIndex, 
          targetItemIndex,
          contentBlocks 
        } = layoutData;
        
        // For grid item moves, rebuild the complete layout since grid content has changed
        console.log('🔍 Grid item moved, rebuilding layout with new structure');
        rebuildCompleteLayout(contentBlocks);
      };
      
      // Enhanced PostMessage listener with update caching
      const handleMessage = (event: MessageEvent) => {
        console.log('📨 PostMessage received in Page.tsx:', event.data);
        
        if (event.data.type === 'HOVER_BLOCK') {
          // Handle block hover highlighting
          const blockId = event.data.payload?.blockId;
          if (blockId) {
            console.log('🎯 Highlighting block in preview:', blockId);
            highlightBlock(blockId);
          }
        } else if (event.data.type === 'UNHOVER_BLOCK') {
          // Handle block unhover
          const blockId = event.data.payload?.blockId;
          if (blockId) {
            console.log('🎯 Removing highlight from block in preview:', blockId);
            unhighlightBlock(blockId);
          }
        } else if (event.data.type === 'HIGHLIGHT_INSERT_POSITION') {
          // Handle insert position highlighting
          const { insertIndex, isGridPosition } = event.data.payload || {};
          if (insertIndex !== undefined) {
            console.log('🎯 Highlighting insert position in preview:', insertIndex, isGridPosition ? '(grid)' : '(root)');
            highlightInsertPosition(insertIndex, isGridPosition);
          }
        } else if (event.data.type === 'UNHIGHLIGHT_INSERT_POSITION') {
          // Handle insert position unhighlighting
          console.log('🎯 Removing insert position highlight in preview');
          unhighlightInsertPosition();
        } else if (event.data.type === 'UPDATE_LAYOUT') {
          // Handle layout reordering updates FIRST - these don't have blockId/updates
          console.log('🔄 Processing layout update:', event.data);
          const layoutData = event.data.payload || event.data;
          
          if (layoutData.type === 'REORDER_BLOCKS') {
            console.log('🔄 Reordering root-level blocks with data:', layoutData);
            handleBlockReordering(layoutData);
          } else if (layoutData.type === 'REORDER_GRID_COLUMNS') {
            console.log('🔄 Reordering grid columns');
            handleGridColumnReordering(layoutData);
          } else if (layoutData.type === 'MOVE_GRID_ITEM') {
            console.log('🔄 Moving grid item');
            handleGridItemMove(layoutData);
          } else if (layoutData.type === 'UPDATE_GRID_COLUMN_FRACTION') {
            console.log('🔄 Updating grid column fraction');
            // For grid column fraction updates, just rebuild with the new content blocks
            if (layoutData.contentBlocks) {
              rebuildCompleteLayout(layoutData.contentBlocks);
            }
          } else if (layoutData.type === 'ADD_BLOCK') {
            console.log('🆕 New block added – rebuilding layout');
            if (layoutData.contentBlocks) {
              rebuildCompleteLayout(layoutData.contentBlocks);
              console.log('✅ Layout rebuilt with new block:', layoutData.newBlockId, 'of type:', layoutData.blockType);
            }
          } else if (layoutData.type === 'GRID_LAYOUT_CHANGE') {
            console.log('🔄 Generic grid layout change – rebuilding layout');
            if (layoutData.contentBlocks) {
              rebuildCompleteLayout(layoutData.contentBlocks);
            }
          } else {
            console.log('⚠️ Unknown layout update type:', layoutData.type, layoutData);
          }
        } else if (event.data.type === 'SIMPLE_TEST' || event.data.type === 'UPDATE_CONTENT') {
          console.log('✅ PostMessage recognized!', event.data);
          
          // Extract data from different message formats
          const blockId = event.data.blockId || event.data.payload?.blockId;
          const updates = event.data.updates || event.data.payload?.updates;
          
          console.log('🔍 Extracted blockId:', blockId);
          console.log('🔍 Extracted updates:', updates);
          
          if (blockId && updates) {
            // Cache all updates for this block (not just content)
            if (!blockUpdateCache.has(blockId)) {
              blockUpdateCache.set(blockId, {});
            }
            
            // Merge the new updates with cached updates
            const cachedUpdates = blockUpdateCache.get(blockId);
            Object.assign(cachedUpdates, updates);
            
            console.log('📦 Cached updates for block:', blockId, cachedUpdates);
            
            // Only proceed with DOM update if we have content
            if (cachedUpdates.content) {
              console.log('🎯 Content found in cache, proceeding with DOM update');
              
              // Debug: Show all elements with data-block-id before searching
              const allBlockElements = document.querySelectorAll('[data-block-id]');
              console.log('🔍 All elements with data-block-id:', Array.from(allBlockElements).map(el => ({
                id: el.getAttribute('data-block-id'),
                tagName: el.tagName,
                className: el.className,
                text: el.textContent?.substring(0, 50) + '...'
              })));
              
              // Try to find the element - first by data-block-id on the wrapper div
              let element = document.querySelector(`[data-block-id="${blockId}"]`);
              console.log('🔍 Found wrapper element with data-block-id:', element);
              
              // If not found on wrapper, try to find it on the content div
              if (!element) {
                // In BlockRenderer, the content is wrapped in a div with data-block-id
                element = document.querySelector(`.text-block[data-block-id="${blockId}"], .heading-block[data-block-id="${blockId}"], .subheading-block[data-block-id="${blockId}"]`);
                console.log('🔍 Found element with class selector:', element);
              }
              
              if (element) {
                console.log('📍 Element details:');
                console.log('   - tagName:', element.tagName);
                console.log('   - className:', element.className);
                console.log('   - innerHTML preview:', element.innerHTML.substring(0, 200));
                
                // Determine content type and update accordingly
                // Check for wrapper classes first (block-wrapper--type)
                if (element.classList.contains('block-wrapper--text') || element.classList.contains('text-block')) {
                  // For text blocks, find the div with dangerouslySetInnerHTML or first div
                  const contentDiv = element.querySelector('div');
                  if (contentDiv) {
                    console.log('✏️ Updating text block content with:', cachedUpdates.content);
                    // Extract content from multi-language object or use string directly
                    let textContent;
                    if (typeof cachedUpdates.content === 'object' && cachedUpdates.content !== null) {
                      // Multi-language content - try nl first, fallback to en, then first available
                      textContent = cachedUpdates.content.nl || cachedUpdates.content.en || Object.values(cachedUpdates.content)[0] || '';
                    } else {
                      textContent = cachedUpdates.content || '';
                    }
                    
                    // Convert markdown links to HTML like the original component does
                    const htmlContent = textContent.replace(/\[([^\]]+)\]\(([^)]+)\)/g, '<a href="$2">$1</a>');
                    contentDiv.innerHTML = htmlContent;
                    console.log('✅ Text block updated successfully!');
                  } else {
                    console.log('❌ No content div found in text block');
                  }
                } else if (element.classList.contains('block-wrapper--title') || element.classList.contains('heading-block')) {
                  // For title/heading blocks, update the h1
                  const heading = element.querySelector('h1');
                  if (heading) {
                    console.log('✏️ Updating heading with:', cachedUpdates.content);
                    // Extract content from multi-language object or use string directly
                    let textContent;
                    if (typeof cachedUpdates.content === 'object' && cachedUpdates.content !== null) {
                      textContent = cachedUpdates.content.nl || cachedUpdates.content.en || Object.values(cachedUpdates.content)[0] || '';
                    } else {
                      textContent = cachedUpdates.content || '';
                    }
                    heading.textContent = textContent;
                    console.log('✅ Heading updated successfully!');
                  } else {
                    console.log('❌ No h1 found in heading block');
                  }
                } else if (element.classList.contains('block-wrapper--subheading') || element.classList.contains('subheading-block')) {
                  // For subheading blocks, update the h2
                  const subheading = element.querySelector('h2');
                  if (subheading) {
                    console.log('✏️ Updating subheading with:', cachedUpdates.content);
                    // Extract content from multi-language object or use string directly
                    let textContent;
                    if (typeof cachedUpdates.content === 'object' && cachedUpdates.content !== null) {
                      textContent = cachedUpdates.content.nl || cachedUpdates.content.en || Object.values(cachedUpdates.content)[0] || '';
                    } else {
                      textContent = cachedUpdates.content || '';
                    }
                    subheading.textContent = textContent;
                    console.log('✅ Subheading updated successfully!');
                  } else {
                    console.log('❌ No h2 found in subheading block');
                  }
                } else if (element.classList.contains('block-wrapper--caption') || element.classList.contains('caption-block')) {
                  // For caption blocks, find the div with dangerouslySetInnerHTML or first div
                  const contentDiv = element.querySelector('div');
                  if (contentDiv) {
                    console.log('✏️ Updating caption block content with:', cachedUpdates.content);
                    // Extract content from multi-language object or use string directly
                    let textContent;
                    if (typeof cachedUpdates.content === 'object' && cachedUpdates.content !== null) {
                      // Multi-language content - try nl first, fallback to en, then first available
                      textContent = cachedUpdates.content.nl || cachedUpdates.content.en || Object.values(cachedUpdates.content)[0] || '';
                    } else {
                      textContent = cachedUpdates.content || '';
                    }
                    
                    // Convert markdown links to HTML like the original component does
                    const htmlContent = textContent.replace(/\[([^\]]+)\]\(([^)]+)\)/g, '<a href="$2">$1</a>');
                    contentDiv.innerHTML = htmlContent;
                    console.log('✅ Caption block updated successfully!');
                  } else {
                    console.log('❌ No content div found in caption block');
                  }
                } else {
                  console.log('⚠️ Unknown block type for element with classes:', element.className);
                  // Fallback: try to find any text element
                  const textElement = element.querySelector('div, h1, h2, h3, p');
                  if (textElement) {
                    console.log('✏️ Fallback: Updating text element with:', cachedUpdates.content);
                    // Extract content from multi-language object or use string directly
                    let textContent;
                    if (typeof cachedUpdates.content === 'object' && cachedUpdates.content !== null) {
                      textContent = cachedUpdates.content.nl || cachedUpdates.content.en || Object.values(cachedUpdates.content)[0] || '';
                    } else {
                      textContent = cachedUpdates.content || '';
                    }
                    textElement.textContent = textContent;
                    console.log('✅ Fallback update successful!');
                  } else {
                    console.log('❌ No suitable text element found for fallback');
                  }
                }
              } else {
                console.log('❌ No element found with data-block-id:', blockId);
              }
            } else {
              console.log('⏳ No content in updates yet, waiting for content update...');
            }
          } else {
            console.log('❌ Missing blockId or updates in message');
          }
        } else {
          console.log('⚠️ Unknown message type:', event.data.type);
        }
      };
      
      window.addEventListener('message', handleMessage);
      
      // Add CSS animations and styles for preview highlights - REACT-FRIENDLY VERSION
      const style = document.createElement('style');
      style.textContent = `
        .live-highlight {
          outline: 2px solid rgba(59, 130, 246, 0.8) !important;
          outline-offset: 2px;
          background-color: rgba(59, 130, 246, 0.1) !important;
          border-radius: 4px;
          transition: all 0.2s ease;
        }
        
        @keyframes insertHighlightPulse {
          0%, 100% { 
            opacity: 0.8; 
          }
          50% { 
            opacity: 1; 
          }
        }
        
        .insert-position-highlight-line {
          /* Styles are applied inline for dynamic positioning */
        }

        
        /* Ensure proper positioning for insert highlights */
        .content-blocks-container {
          position: relative;
        }
      `;
      document.head.appendChild(style);
      
      // Clean up on unmount - REACT-FRIENDLY VERSION
      return () => {
        window.removeEventListener('message', handleMessage);
        previewListener.cleanup();
        
        // Clean up CSS classes and elements
        document.querySelectorAll('.live-highlight').forEach(el => {
          el.classList.remove('live-highlight');
        });
        
        // Clean up insert position highlight lines
        document.querySelectorAll('.insert-position-highlight-line').forEach(el => {
          if (el.parentNode) {
            el.parentNode.removeChild(el);
          }
        });
        
        document.querySelectorAll('.grid-insert-highlight').forEach(el => {
          el.classList.remove('grid-insert-highlight');
        });
        
        // Remove the style element safely (only remaining DOM operation)
        if (style && style.parentNode) {
          try {
            style.parentNode.removeChild(style);
          } catch (error) {
            console.debug('Cleanup: Style element already removed:', error);
          }
        }
      };
    }
  }, [isPreviewMode]);

  useEffect(() => {
    const fetchPageData = async () => {
      // Don't start loading page data until layout settings are ready
      if (layoutLoading) {
        return;
      }

      setIsLoading(true);
      setError(null);
      setPageData(null);
      setUsingFallback(false);

      // Only log in development mode
      if (import.meta.env.DEV) {
        console.log(`Fetching data for ${pageType} page (${pageIdentifier})${user ? ' (authenticated)' : ' (unauthenticated)'}`);
      }

      try {
        let data: PageData | null = null;

        // Fetch data based on page type
        switch (pageType) {
          case 'main':
            data = await apiClient.getPage('main', currentLanguage, versionId || undefined);
            break;
          case 'clients':
            data = await apiClient.getPage('clients', currentLanguage, versionId || undefined);
            break;
          case 'cases':
            data = await apiClient.getPage('cases', currentLanguage, versionId || undefined);
            break;
          case 'customPage':
            if (!pageIdentifier) throw new Error('No page identifier provided');
            data = await apiClient.getPage(pageIdentifier, currentLanguage, versionId || undefined);
            break;
          case 'hierarchicalPage':
            if (!pageIdentifier) throw new Error('No page identifier provided');
            // For hierarchical pages, use the child identifier to fetch the page
            data = await apiClient.getPage(pageIdentifier, currentLanguage, versionId || undefined);
            break;
          case 'case':
            if (!pageIdentifier) throw new Error('No case identifier provided');
            // Cases are now pages - use pages API directly
            data = await apiClient.getPage(pageIdentifier, currentLanguage, versionId || undefined);
            break;
          default:
            throw new Error(`Unknown page type: ${pageType}`);
        }

        if (!data) {
          throw new Error(`No data available for ${pageType} page`);
        }

        // Check if page is hidden and should redirect
        if ((data as any).status === 'HIDDEN' || (data as any).versionStatus === 'HIDDEN' || (data as any).isHidden) {
          // Try to get redirect URL from the page data first, then from redirects API
          const redirectUrl = (data as any).redirectUrl;
          
          if (redirectUrl) {
            if (import.meta.env.DEV) {
              console.log(`Page is hidden, redirecting to: ${redirectUrl}`);
            }
            navigate(redirectUrl, { replace: true });
            return;
          }
          
          // Fallback: try to get redirect URL from redirects API
          try {
            const currentPath = location.pathname;
            const redirect = await apiClient.getRedirect(currentPath);
            if (redirect && redirect.redirect && redirect.redirect.toPath) {
              if (import.meta.env.DEV) {
                console.log(`Page is hidden, redirecting via API to: ${redirect.redirect.toPath}`);
              }
              navigate(redirect.redirect.toPath, { replace: true });
              return;
            }
          } catch (error) {
            if (import.meta.env.DEV) {
              console.log('No redirect found for hidden page via API');
            }
          }
          
          // If no redirect URL is found, show 404
          if (import.meta.env.DEV) {
            console.log('No redirect found for hidden page, showing 404');
          }
          setError('Page not found');
          return;
        }

        // Basic validation of overview data
        if (data.overview && !Array.isArray(data.overview)) {
          if (import.meta.env.DEV) {
            console.warn('Fetched overview data is not an array, attempting to parse or fallback.');
          }
          data.overview = [];
        }

        setPageData(data);

      } catch (err: any) {
        console.error(`Error fetching ${pageType} page data:`, err);
        
        // Handle specific error cases
        if (err.message?.includes('404') || err.status === 404) {
          // For 404 errors, show appropriate message based on authentication status
          if (user) {
            setError(`${pageType === 'case' ? 'Case' : pageType === 'customPage' ? 'Page' : 'Page'} niet gevonden. Mogelijk is deze verwijderd of heeft de URL een typfout.`);
          } else {
            setError(`${pageType === 'case' ? 'Case' : pageType === 'customPage' ? 'Page' : 'Page'} niet gevonden.`);
          }
        } else {
          setError(`Could not load ${pageType === 'customPage' ? 'custom page' : pageType} page: ${err.message || 'Unknown error'}`);
        }

        // Cases are now pages - no fallback data needed
        // All pages either exist in the database or show 404
      } finally {
        setIsLoading(false);
      }
    };

    fetchPageData();
  }, [pageType, pageIdentifier, user, layoutLoading, currentLanguage, versionId, location.pathname]);

  // Separate useEffect to handle title localization - runs whenever pageData changes
  useEffect(() => {
    if (!pageData) return;
    
    // Use localized title for the document title
    let localizedTitle = getLocalizedContent(pageData.title, currentLanguage);
    
    // Quick fix for English page titles - hardcode known English titles
    if (currentLanguage === 'en' && pageData.identifier === 'main') {
      localizedTitle = 'Glad to have you here';
    }
    
    document.title = `${localizedTitle} - Studio Paulides`;
  }, [pageData, currentLanguage]);

  // Memoize content analysis
  const contentAnalysis = useMemo(() => {
    if (!pageData?.overview) {
      return { hasTitleBlock: false, hasH1InContent: false, needsMainHeading: true };
    }
    
    const hasTitleBlock = pageData.overview.some(block => block.type === 'title');
    const firstTextBlock = pageData.overview.find(block => block.type === 'text') as any;
    const localizedContent = firstTextBlock ? getLocalizedContent(firstTextBlock.content, currentLanguage) : '';
    const hasH1InContent = localizedContent && typeof localizedContent === 'string' && localizedContent.includes('<h1');
    
    return {
      hasTitleBlock,
      hasH1InContent,
      needsMainHeading: !hasTitleBlock && !hasH1InContent
    };
  }, [pageData?.overview, currentLanguage]);

  // Preload hero images for better LCP performance
  useHeroImagePreload(pageData?.overview || []);

  // Render loading state with proper layout structure
  // Show loading when layout is loading OR when page data is loading
  if (layoutLoading || isLoading) {
    return (
      <main className="main-content-area">
        <LayoutDebugger pageId={pageIdentifier} />
        <article className="content-section">
          {/* Use the same container structure as real content */}
          <div className="content-blocks-container">
            
            {/* Title skeleton */}
            <div className="skeleton-heading" style={{
              height: '3rem',
              backgroundColor: 'var(--color-z-30)',
              borderRadius: '8px',
              width: '80%',
              opacity: '0.5',
              animation: 'pulse 4.5s ease-in-out infinite alternate',
              animationDelay: '0.1s',
              willChange: 'opacity'
            }}></div>
            
            {/* Content blocks skeleton - using the same gap as real content */}
            {[1, 2, 3].map((i) => (
              <div key={i} style={{
                height: '8rem',
                backgroundColor: 'var(--color-z-30)',
                borderRadius: '8px',
                opacity: '0.5',
                animation: 'pulse 4.5s ease-in-out infinite alternate',
                animationDelay: `${i * 0.1}s`,
                willChange: 'opacity'
              }}></div>
            ))}
          </div>
          
          <style>{`
            @keyframes pulse {
              0% { opacity: 0.3; }
              100% { opacity: 0.7; }
            }
            
            @media (prefers-color-scheme: dark) {
              .skeleton-loader div {
                background-color: var(--color-z-90) !important;
                opacity: 0.5 !important;
              }
            }
          `}</style>
        </article>
      </main>
    );
  }

  // Render error state
  if (error) {
    // Check if this is a 404 error
    const is404Error = error.includes('404') || error.includes('niet gevonden') || error.includes('not found');
    
    if (is404Error) {
      return <NotFoundPage />;
    }
    
    return (
      <main className="main-content-area">
        <LayoutDebugger pageId={pageIdentifier} />
        <article 
          className="content-section" 
          style={{ 
            paddingTop: 'var(--spacing-5)',
            paddingLeft: '1rem', 
            paddingRight: '1rem',
            textAlign: 'center'
          }} 
        >
          <div className="error-state">
            <div style={{
              fontSize: '3rem',
              marginBottom: '1rem',
              opacity: 0.5
            }}>⚠️</div>
            <h1 className="main-heading" style={{ marginBottom: '1rem' }}>
              Er ging iets mis
            </h1>
            <p className="section-text" style={{ 
              marginBottom: '2rem',
              color: 'var(--color-z-70)'
            }}>
              We konden de pagina niet laden. Dit kan tijdelijk zijn door een verbindingsprobleem.
            </p>
            <div style={{ marginBottom: '2rem' }}>
              <button 
                onClick={() => window.location.reload()} 
                className="button button--primary"
                style={{ marginRight: '1rem' }}
              >
                Probeer opnieuw
              </button>
              <button 
                onClick={() => window.history.back()} 
                className="button button--secondary"
              >
                Ga terug
              </button>
            </div>
          </div>
        </article>
      </main>
    );
  }

  // Render content
  return (
    <main className="main-content-area">
      <LayoutDebugger pageId={pageIdentifier} />
      <article className="content-section" role="main">
        <div className="content-blocks-container">
          {/* Render main heading if needed */}
          {contentAnalysis.needsMainHeading && pageData && (
            <h1 className="main-heading">{pageData.title}</h1>
          )}
          
          {/* Render content blocks with performance optimizations */}
          {pageData?.overview && (
            <PerformanceOptimizedRenderer
              blocks={pageData.overview}
              currentLanguage={currentLanguage as any}
              priorityBlockCount={3}
            />
          )}
          
          {/* Show fallback notice if using fallback data */}
          {usingFallback && (
            <div style={{
              padding: '1rem',
              backgroundColor: 'var(--color-warning-bg, #fff3cd)',
              border: '1px solid var(--color-warning-border, #ffeaa7)',
              borderRadius: '8px',
              marginTop: '2rem',
              fontSize: '0.9rem',
              color: 'var(--color-warning-text, #856404)'
            }}>
              <strong>⚠️ Fallback Data:</strong> Deze case wordt getoond met lokale fallback data omdat de server data niet beschikbaar is.
            </div>
          )}
        </div>
      </article>
    </main>
  );
};

export default Page;